from bloonix.heaven import H
from bloonix.heaven.plugin.crud import CRUD

class Controller(CRUD):
    def startup(self):
        self.db = H.model.db.contactgroup
        self.add_crud_routes("/config/contactgroups")
        H.route(
            "/config/contactgroups/:id/contacts/member",
            "contactgroup_contact_member")
        H.route(
            "/config/contactgroups/:id/contacts/no-member",
            "contactgroup_contact_no_member")
        H.route(
            "/config/contactgroups/:id/contacts/:contact_id/add",
            "contactgroup_contact_add")
        H.route(
            "/config/contactgroups/:id/contacts/:contact_id/remove",
            "contactgroup_contact_remove")
        H.route(
            "/config/contactgroups/:id/hosts/member",
            "contactgroup_host_member")
        H.route(
            "/config/contactgroups/:id/hosts/no-member",
            "contactgroup_host_no_member")
        H.route(
            "/config/contactgroups/:id/hosts/:host_id/add",
            "contactgroup_host_add")
        H.route(
            "/config/contactgroups/:id/hosts/:host_id/remove",
            "contactgroup_host_remove")
        H.route(
            "/config/contactgroups/:id/services/member",
            "contactgroup_service_member")
        H.route(
            "/config/contactgroups/:id/services/no-member",
            "contactgroup_service_no_member")
        H.route(
            "/config/contactgroups/:id/services/:service_id/add",
            "contactgroup_service_add")
        H.route(
            "/config/contactgroups/:id/services/:service_id/remove",
            "contactgroup_service_remove")

    def auto(self, opts):
        if "id" in opts:
            contactgroup = H.model.db.contactgroup.get(opts["id"])
            if not contactgroup:
                return H.plugin.error.set("err-600")
            H.cache["contactgroup"] = contactgroup
        if "host_id" in opts:
            if not H.model.db.host.get(opts["host_id"]):
                return H.plugin.error.set("err-600")
        if "service_id" in opts:
            if not H.model.db.service.get(opts["service_id"]):
                return H.plugin.error.set("err-600")
        return True

    def list(self, opts):
        ret = H.model.db.contactgroup.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            count="id",
            table=["contactgroup", ["id", "name", "description"]],
            order=["asc", "contactgroup.name"])

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_contact_member(self, opts):
        ret = H.model.db.contact.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            count="id",
            table=["contact", ["id", "name"]],
            join=[
                "inner", [
                    "contact_contactgroup",
                    "contact.id",
                    "contact_contactgroup.contact_id"
                ]
            ],
            condition=[
                "where", [
                    "contact_contactgroup.contactgroup_id",
                    opts["id"]
                ]
            ],
            order=["asc", "contact.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_contact_no_member(self, opts):
        ret = H.model.db.contact.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            count="id",
            table=["contact", ["id", "name"]],
            condition=[
                "where", ["contact.id", "not in", {
                    "table": "contact_contactgroup",
                    "columns": "contact_id",
                    "condition": [
                        "where", [
                            "contact_contactgroup.contactgroup_id",
                            opts["id"]
                        ]
                    ]
                }]
            ],
            order=["asc", "contact.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_contact_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.contact_contactgroup.create({
            "contactgroup_id": opts["id"],
            "contact_id": opts["contact_id"]
        });

    def contactgroup_contact_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.contact_contactgroup.delete_by_col(
            "contactgroup_id", opts["id"],
            "contact_id", opts["contact_id"]
        )
    def contactgroup_host_member(self, opts):
        ret = H.model.db.host.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            count="id",
            table=["host", ["id", "hostname", "ipaddr1", "project_class", "location_class"]],
            join=[
                "inner", [
                    "host_contactgroup",
                    "host.id",
                    "host_contactgroup.host_id"
                ]
            ],
            condition=[
                "where", [
                    "host_contactgroup.contactgroup_id",
                    opts["id"]
                ]
            ],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_host_no_member(self, opts):
        ret = H.model.db.host.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            count="id",
            table=["host", ["id", "hostname", "ipaddr1", "project_class", "location_class"]],
            condition=[
                "where", ["host.id", "not in", {
                    "table": "host_contactgroup",
                    "columns": "host_id",
                    "condition": [
                        "where", [
                            "host_contactgroup.contactgroup_id",
                            opts["id"]
                        ]
                    ]
                }]
            ],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_host_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.host_contactgroup.create({
            "contactgroup_id": opts["id"],
            "host_id": opts["host_id"]
        });

    def contactgroup_host_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.host_contactgroup.delete_by_col(
            "contactgroup_id", opts["id"],
            "host_id", opts["host_id"]
        )

    def contactgroup_service_member(self, opts):
        ret = H.model.db.service.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            table=[
                "service", "id",
                "service_parameter", "service_name",
                "host", "hostname"
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id"
                ],
                "inner", [
                    "service_contactgroup",
                    "service.id",
                    "service_contactgroup.service_id"
                ]
            ],
            condition=[
                "where", [
                    "service_contactgroup.contactgroup_id",
                    opts["id"]
                ]
            ],
            order=[
                "asc", "host.hostname",
                "asc", "service_parameter.service_name"
            ]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_service_no_member(self, opts):
        ret = H.model.db.service.query(
            parse_int=["id"],
            query=H.req.get_query_data(),
            table=[
                "service", "id",
                "service_parameter", "service_name",
                "host", "hostname"
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id"
                ]
            ],
            condition=[
                "where", ["service.id", "not in", {
                    "table": "service_contactgroup",
                    "columns": "service_id",
                    "condition": [
                        "where", [
                            "service_contactgroup.contactgroup_id",
                            opts["id"]
                        ]
                    ]
                }]
            ],
            order=[
                "asc", "host.hostname",
                "asc", "service_parameter.service_name"
            ]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroup_service_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.service_contactgroup.create({
            "contactgroup_id": opts["id"],
            "service_id": opts["service_id"]
        });

    def contactgroup_service_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.service_contactgroup.delete_by_col(
            "contactgroup_id", opts["id"],
            "service_id", opts["service_id"]
        )
