from bloonix.heaven import H
import re

class Controller:
    def startup(self):
        self.db = H.model.db.contact_message_service
        H.route("/config/contacts/:id/message-services", "list")
        H.route("/config/contacts/:id/message-services/add", "add")
        H.route("/config/contacts/:id/message-services/:cms_id/update", "update")
        H.route("/config/contacts/:id/message-services/:cms_id/remove", "remove")
        H.route("/config/contacts/:id/message-services/:cms_id/options", "options")
        H.route("/config/contacts/:id/message-services/options", "options")

    def auto(self, opts):
        if "cms_id" in opts:
            value = self.db.by_id_and_contact_id(opts["cms_id"], opts["id"])
            if not value:
                return H.plugin.error.set("err-600")
            H.cache["contact_message_service"] = value
        return True

    def list(self, opts):
        H.stash.data = self.db.by_contact_id(opts["id"])

    def options(self, opts):
        self.db.init()

        H.stash.data["options"] = self.db.validator.get_options()

        if "cms_id" in opts:
            H.stash.data["values"] = self.db.get(opts["cms_id"])
        else:
            H.stash.data["values"] = self.db.validator.get_defaults()

    def add(self, opts):
        if not H.plugin.token.check():
            return

        self.db.init()
        res = self.db.validator.validate()

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        res.data["contact_id"] = opts["id"]
        ret = self.db.create_and_get(res.data)

        H.plugin.audit.create(
            target="contact_message_service",
            new_data=res.data
        )

    def update(self, opts):
        if not H.plugin.token.check():
            return

        self.db.init()
        res = self.db.validator.validate(ignore_missing=True)

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        self.db.update_by_id(opts["cms_id"], res.data)

        H.plugin.audit.update(
            target="contact_message_service",
            old_data=H.cache["contact_message_service"],
            new_data=res.data
        )

    def remove(self, opts):
        if not H.plugin.token.check():
            return

        self.db.delete_by_id(opts["cms_id"])

        H.plugin.audit.delete(
            target="contact_message_service",
            old_data=H.cache["contact_message_service"]
        )
