from bloonix.exception import BloonixTooMuchObjects
from bloonix.heaven import H
import bloonix.utils as utils
import re

class Controller:
    def startup(self):
        H.route("/config/hosts/:id/services", "list")
        H.route("/config/hosts/:id/services/create/:command", "create")
        H.route("/config/hosts/:id/services/:service_id", "show")
        H.route("/config/hosts/:id/services/:service_id/update", "update")
        H.route("/config/hosts/:id/services/:service_id/delete", "delete")
        H.route("/config/hosts/:id/services/:service_id/options", "options")
        H.route("/config/hosts/:id/services/options/:command", "options")
        H.route("/config/hosts/:id/services/:service_id/variables", "variables")
        H.route("/config/hosts/:id/services/:service_id/variables/update", "update_variables")

    def auto(self, opts):
        if "service_id" in opts:
            service = H.model.db.service.by_id_and_by_host_id(
                opts["service_id"], opts["id"])
            if not service:
                return H.plugin.error.set("err-600")
        return H.plugin.service.auto(opts)

    def show(self, opts):
        return H.plugin.service.show(opts)

    def list(self, opts):
        ret = H.model.db.service.query(
            query=H.req.get_query_data(),
            count="id",
            table=[
                "service", [
                    "id",
                    "service_parameter_id",
                    "status",
                    "enabled",
                    "notification_enabled",
                    "acknowledged"
                ],
                "service_parameter", [
                    "service_name",
                    "command",
                    "remote_check",
                    "interval",
                    "retry_interval",
                    "attempt_max",
                    "host_alive_check"
                ],
                "template", [
                    "id AS template_id",
                    "name AS template_name"
                ]
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id",
                ],
                "left", [
                    "template",
                    "service_parameter.template_id",
                    "template.id",
                ]
            ],
            condition=["where", ["service.host_id", opts["id"]]],
            order=["desc", "status", "asc", "service_name"]
        )
        if ret.success == False:
            return H.plugin.error.set("err-606")
        H.stash.data = ret.data
        H.stash.total = ret.total

    def options(self, opts):
        #if H.cache["service_parameter"]["template_id"]:
        #    return H.plugin.error.set("err-600")
        H.plugin.service.options(opts)

    def create(self, opts):
        try:
            H.plugin.service.create(opts)
        except BloonixTooMuchObjects:
            return H.plugin.error.set("err-800")

    def update(self, opts):
        if H.cache["service_parameter"]["template_id"]:
            return H.plugin.error.set("err-802")
        H.plugin.service.update(opts)

    def delete(self, opts):
        if H.cache["service_parameter"]["template_id"]:
            return H.plugin.error.set("err-801")
        H.plugin.service.delete(opts)

    def variables(self, opts):
        H.stash.data = self._variables(opts)

    def _variables(self, opts):
        # Filter relevant variables for the service
        ret = {}
        variables = {}
        all_variables = {}

        template = H.model.db.template.get(H.cache["service_parameter"]["template_id"])
        variables["template"] = template["variables"]
        all_variables.update(variables["template"])

        variables["host"] = H.cache["host"]["variables"]
        all_variables.update(variables["host"])

        variables["service"] = H.cache["service"]["variables"]
        all_variables.update(variables["service"])

        command_options = H.cache["service_parameter"]["command_options"]
        for opt in command_options:
            if isinstance(command_options[opt], list):
                for value in command_options[opt]:
                    if isinstance(value, list):
                        for v in value:
                            self._in(all_variables, variables, v, ret)
                    else:
                        self._in(all_variables, variables, value, ret)
            elif isinstance(command_options[opt], str):
                self._in(all_variables, variables, command_options[opt], ret)

        return ret

    def _in(self, all_variables, variables, value, ret):
        for var in all_variables:
            if re.search(r"%%%s%%" % var, value):
                if var in ret:
                    continue
                ret[var] = {}
                if var in variables["template"]:
                    ret[var]["template"] = variables["template"][var]
                if var in variables["host"]:
                    ret[var]["host"] = variables["host"][var]
                if var in variables["service"]:
                    ret[var]["service"] = variables["service"][var]

    def update_variables(self, opts):
        if not H.plugin.token.check():
            return
        variables = self._variables(opts)
        pv = H.req.data.get("variables", {})
        err = []
        for key in pv:
            if key not in variables or not isinstance(pv[key], (int, str)):
                err.append(key)
        if len(err):
            return H.plugin.error.set("err-711", failed=err)
        H.model.db.service.update_by_id(
            opts["service_id"],
            {"variables": pv},
        )
