import re
import time
import bloonix.utils as utils
from bloonix.heaven import H
from bloonix.heaven.validator import Validator

class Controller:
    def startup(self):
        H.route("/hosts/:id/report", "report")
        H.route("/hosts/:id/report/:service_id", "report")

    def report(self, opts):
        v = Validator()
        v.set({
            "from_time": {
                "type": str,
                "isa_unixtime": H.user["timezone"],
            },
            "to_time": {
                "type": str,
                "isa_unixtime": H.user["timezone"],
                "depends_on": {
                    "from_time": {
                        "lower_or_greater": "greater",
                    }
                }
            },
        })
        res = v.validate()
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        from_time = res.data["from_time"]
        to_time = res.data["to_time"]
        curtime = int(time.time())

        if from_time > curtime:
            from_time = curtime
        if to_time > curtime:
            to_time = curtime

        services = H.model.db.service.get_status_since(opts["id"], opts.get("service_id", None))
        for service in services:
            self.calculate_availability(service, from_time, to_time)

        if "service_id" in opts:
            H.stash.data = services[0]
        else:
            H.stash.data = services

    def calculate_availability(self, service, from_time, to_time):
        events = H.model.db.event.by_service_id(service["id"], from_time, to_time)
        total_time = to_time - from_time
        a = {}
        d = {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0}
        s = {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0}
        t = {"downtime": 0, "flapping": 0}
        n = {}
        r = {
            "b15": {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0},
            "b30": {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0},
            "b60": {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0},
            "b180": {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0},
            "bx": {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0},
        }
        for i in range(24):
            n[i] = {0:0, 1:0, 2:0, 3:0, 4:0, 5:0, 6:0}

        # If there are no events in this period, then the first event
        # after this time must be selected. If there is then also no
        # result, then the status of the service can be used.
        if len(events) == 0:
            event = H.model.db.event.min_by_service_id(service["id"], to_time)
            if event["min"] is not None:
                event = H.model.db.event.find("service_id", service["id"], "time", event["min"])
                # check if the event is between from_time and to_time
                if event["time"] - event["duration"] < to_time:
                    d[event["last_status"]] += to_time - event["time"] - event["duration"]
            elif service["status_since"] < from_time:
                d[service["status"]] = total_time
            elif service["status_since"] < to_time:
                d[service["status"]] = to_time - service["status_since"]
        else:
            for event in events:
                if event["status"] == event["last_status"]:
                    continue
                if event["duration"] < 900:
                    r["b15"][event["last_status"]] += 1
                elif event["duration"] < 1800:
                    r["b30"][event["last_status"]] += 1
                elif event["duration"] < 3600:
                    r["b60"][event["last_status"]] += 1
                elif event["duration"] < 10800:
                    r["b180"][event["last_status"]] += 1
                else:
                    r["bx"][event["last_status"]] += 1

                if event["tags"] is not None:
                    for tag in re.split(r",", event["tags"]):
                        if not len(tag):
                            continue
                        if tag not in t:
                            t[tag] = 0
                        t[tag] += 1
                s[event["status"]] += 1
                hour = int(utils.timestamp(unixtime=event["time"], time_format="%H"))
                n[hour][event["status"]] += 1

                if event["time"] - event["duration"] < from_time:
                    d[event["last_status"]] += event["time"] - from_time
                else:
                    d[event["last_status"]] += event["duration"]

            d[events[-1]["status"]] += to_time - events[-1]["time"]

        _total_time = d[0] + d[1] + d[2] + d[3] + d[4] + d[5] + d[6]
        if _total_time < total_time:
            d[1] += total_time - _total_time

        _mp = 100
        for status in d:
            a[status] = float("%.2f" % (100 * d[status] / total_time))
            if _mp - a[status] < 0:
                a[status] = _mp
            _mp = float("%.2f" % (_mp - a[status]))

        service["total_time"] = total_time
        service["total_number_of_events"] = len(events)
        service["events_by_status"] = s
        service["events_by_hour"] = n
        service["status_duration_in_percent"] = a
        service["number_of_tags"] = t
        service["status_duration"] = d
        service["availability"] = 100 - a[5]
        service["events_by_duration"] = r
