from bloonix.heaven import H
from bloonix.heaven.validator import Validator
from bloonix.netaddr import get_ip_by_hostname
from bloonix import ccodes
from bloonix import utils
from bloonix import rx

class Controller:
    def startup(self):
        H.route("/register", "register")

    def register(self, opts):
        reg = H.model.db.regauth.get_authkey(H.req.param("authkey"))
        if not reg:
            return H.plugin.error.set("err-699")
        data = self.validate(reg["allowed_tags"])
        if not data:
            return
        host = H.model.db.host.create_and_get({
            "hostname": data["hostname"],
            "ipaddr1": data["ipaddr1"]})
        H.model.db.host.delete_by_id(host["id"])
        if (H.model.db.host.by_hostname(data["hostname"])
                or H.model.db.regauth_host.by_hostname(data["hostname"])):
            return H.plugin.error.set("err-602")
        data["host_id"] = host["id"]
        data["password"] = utils.pwgen(64)
        data["regauth_id"] = reg["id"]
        H.model.db.regauth_host.create(data)
        H.stash.data = {"host_id": data["host_id"], "password": data["password"]}

    def validate(self, allowed_tags):
        data = H.req.params([
            "hostname",
            "ipaddr1",
            "ipaddr2",
            "ipaddr3",
            "ipaddr4",
            "sysinfo",
            "country_code",
            "location_class",
            "project_class",
            "tags",
            "variables",
            "allow_from",
        ])

        for key in ["ipaddr1", "ipaddr2", "ipaddr3", "ipaddr4"]:
            if key in data and (data[key] == "v4" or data[key] == "v6"):
                data[key] = get_ip_by_hostname(data["hostname"], data[key])
                if data[key] is None:
                    del data[key]

        v = Validator(get_resultset=True)
        v.set({
            "hostname": {
                "type": str,
                "regex": rx.HOSTNAME
            },
            "ipaddr1": {
                "type": str,
                "regex": rx.IPADDR
            },
            "ipaddr2": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr3": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr3": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr4": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "sysinfo": {
                "type": str,
                "minlength": 0,
                "maxlength": 400,
                "optional": True
            },
            "country_code": {
                "type": str,
                "list_of_dicts_key": "value",
                "list_of_dicts_options": ccodes.country_as_form(),
                "allow_empty": True,
                "optional": True
            },
            "location_class": {
                "type": str,
                "regex": rx.CLASS_PATH,
                "maxlength": 100,
                "default": "",
            },
            "project_class": {
                "type": str,
                "regex": rx.CLASS_PATH,
                "maxlength": 100,
                "default": "",
            },
            "variables": {
                "type": dict,
                "function": utils.validate_pv_dict,
                "allow_empty": True,
                "maxlength": 20000,
                "default": {}
            },
            "allow_from": {
                "type": list,
                "allow_empty": True,
                "maxlength": 10,
                "default": []
            }
        })

        if len(allowed_tags):
            v.set({"tags": {
                "type": list,
                "multioptions": allowed_tags,
                "allow_empty": True,
            }})
        else:
            v.set({"tags": {
                "type": list,
                "maxsize": 50,
                "maxlength": 1000,
                "default": [],
                "regex": r"\w",
                "allow_empty": True
            }})
        res = v.validate(data)

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        return res.data
