from bloonix.dbi.crud import CRUD
from bloonix.heaven import H

class Chart(CRUD):
    def list(self, query_data, host_id=None):
        query = {
            "query": query_data,
            "parse_int": [
                "service_id",
                "host_id",
            ],
            "count": "service.id",
            "table": [
                "service", ["id as service_id"],
                "service_parameter", ["service_name", "command"],
                "plugin", ["service as plugin_service"],
                "chart", [
                    "title as chart_title",
                    "chart_id",
                ],
            ],
            "join": [
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id",
                ],
                "inner", [
                    "plugin",
                    "service_parameter.command",
                    "plugin.command",
                ],
                "inner", [
                    "chart",
                    "plugin.command",
                    "chart.command",
                ],
            ],
        }

        if host_id:
            query["condition"] = ["where", ["service.host_id", host_id]]
            query["order"] = [
                "asc", [
                    "service_parameter.service_name",
                    "plugin.service"
                ]
            ]
        else:
            query["table"][1].append("host_id")
            query["table"].append("host")
            query["table"].append("hostname")
            query["join"].append("inner")
            query["join"].append(["host", "service.host_id", "host.id"])
            query["order"] = [
                "asc", [
                    "host.hostname",
                    "service_parameter.service_name",
                    "plugin.service"
                ]
            ]
            query = H.plugin.role.filter_services(**query)

        return self.query(**query)
