import re
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator, validate
from bloonix.heaven import H
from bloonix.validator.static_vars import ERROR_MESSAGE
from bloonix.json import json

def _form_field_username(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 100,
        "optional": optional,
    }

def _form_field_password(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 100,
        "optional": optional,
    }

def _form_field_userAgent(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 200,
        "optional": optional,
    }

def _form_field_filename(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 200,
        "optional": optional,
    }

def _form_field_url(optional):
    return {
        "type": str,
        "maxlength": 2048,
        "regex": r"https{0,1}://[^\s]+\.[^\s+]+\Z",
        "optional": optional,
    }

def _form_field_parent(optional):
    return {
        "type": str,
        "regex": r"^(#[^\s]+)?\Z",
        "maxlength": 200,
        "optional": optional,
    }

def _form_field_element(optional):
    return {
        "type": str,
        "maxlength": 200,
        "regex": r"^(?:[^\s]+|<\s*[a-zA-Z0-9\-]{1,16}(?:\[\d{1,2}\]){0,1}(?:\s+[a-zA-Z0-9_\-]+=(?:'[^']*'|\"[^\"]*\"))*\s*>)\Z",
        "optional": optional,
    }

def _form_field_value(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 200,
        "optional": optional,
    }

def _form_field_event(optional):
    return {
        "type": str,
        "options": [
            'change', 'keyup', 'keydown', 'keypress',
            'focus', 'mousedown', 'mouseup'
        ],
        "optional": optional,
    }

def _form_field_hidden(optional):
    return {
        "type": bool,
        "options": [0, 1],
        "default": 0,
        "optional": optional,
    }

def _form_field_text(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 200,
        "optional": optional,
    }

def _form_field_ms(optional):
    return {
        "type": int,
        "optional": optional,
    }

def _form_field_name(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 200,
        "optional": optional,
    }

def _form_field_domain(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 100,
        "optional": optional,
    }

def _form_field_status(optional):
    return {
        "type": int,
        "allow_empty": True,
        "optional": optional,
    }

def _form_field_contentType(optional):
    return {
        "type": str,
        "regex": r"[^\s]|^\Z",
        "maxlength": 100,
        "optional": optional,
    }

def _form_field_html(optional):
    return {
        "type": str,
        "regex": r"[^\s]",
        "maxlength": 200,
        "optional": optional,
    }

_wtrm_validate_config_by_action = {
    "doAuth": Validator({
        "username": _form_field_username(False),
        "password": _form_field_password(False),
    }),
    "doUserAgent": Validator({
        "userAgent": _form_field_userAgent(False),
    }),
    "doUrl": Validator({
        "url": _form_field_url(False),
        "status": _form_field_status(False),
        "contentType": _form_field_contentType(True),
    }),
    "doFill": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(False),
        "hidden": _form_field_hidden(True),
        "event": _form_field_event(True),
    }),
    "doTriggerEvent": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "event": _form_field_event(False),
    }),
    "doClick": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
    }),
    "doSubmit": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
    }),
    "doCheck": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "doUncheck": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "doSelect": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "doWaitForElement": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "text": _form_field_text(True),
    }),
    "doSleep": Validator({
        "ms": _form_field_ms(False),
    }),
    "doSwitchToFrame": Validator({
        "name": _form_field_name(False),
    }),
    "doSwitchToParentFrame": None,
    "doSwitchToNewPage": None,
    "doSwitchToMainPage": None,
    "doAddCookie": Validator({
        "name": _form_field_name(False),
        "value": _form_field_value(False),
        "domain": _form_field_domain(False),
    }),
    "checkUrl": Validator({
        "url": _form_field_url(False),
        "status": _form_field_status(False),
        "contentType": _form_field_contentType(True),
    }),
    "checkIfElementExists": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
    }),
    "checkIfElementNotExists": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
    }),
    "checkIfElementHasText": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "text": _form_field_text(False),
    }),
    "checkIfElementHasNotText": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "text": _form_field_text(False),
    }),
    "checkIfElementMatchText": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "text": _form_field_text(False),
    }),
    "checkIfElementHasHTML": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "html": _form_field_html(False),
    }),
    "checkIfElementHasNotHTML": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "html": _form_field_html(False),
    }),
    "checkIfElementHasValue": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(False),
        "hidden": _form_field_hidden(True),
    }),
    "checkIfElementHasNotValue": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(False),
        "hidden": _form_field_hidden(True),
    }),
    "checkIfElementIsChecked": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "checkIfElementIsNotChecked": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "checkIfElementIsSelected": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "checkIfElementIsNotSelected": Validator({
        "parent": _form_field_parent(True),
        "element": _form_field_element(False),
        "value": _form_field_value(True),
    }),
    "checkDownload": Validator({
        "filename": _form_field_filename(False),
    })
}

def _validate_wtrm_workflow(workflow):
    if not len(workflow):
        return False
    has_do_url = False
    for step in workflow:
        if not isinstance(step, dict):
            return False
        if "id" in step:
            del step["id"]
        if "action" not in step or not isinstance(step["action"], str):
            return False
        step = dict(step)
        action = step["action"]
        del step["action"]
        if action == "doUrl":
            has_do_url = True
        if _wtrm_validate_config_by_action[action] == None:
            if len(step):
                return False
            continue
        res = _wtrm_validate_config_by_action[action].validate(step)
        if res.has_failed:
            return False
    return has_do_url


class ServiceParameter(CRUDV):
    def V(self, value):
        H.log.warning(workflow)
        return False

    def __init__(self):
        self.validate_wtrm_actions = _wtrm_validate_config_by_action

    def _vdebug(self, message):
        H.log.debug("@@validation: %s" % message)

    def get_form_values(self, service_type, plugin):
        v = self._init_validator(service_type, plugin)
        return v.get_options(), v.get_defaults()

    def validate_wtrm_workflow(self, workflow):
        return _validate_wtrm_workflow(workflow)

    def validate(self, service_type, plugin, ignore_missing=None):
        v = self._init_validator(service_type, plugin, ignore_missing)
        r = v.validate(ignore_missing=ignore_missing)
        if "command_options" in r.data:
            self._validate_command_options(r, plugin)
        return r

    def _init_validator(self, service_type, plugin, ignore_missing=None):
        if plugin["frequency"] == "mid":
            interval = [600, 900, 1800, 3600, 7200, 14400, 21600, 28800, 43200, 86400]
            interval_default = 600
            retry_interval = [180, 300, 600, 900, 1800, 3600, 7200, 14400]
            retry_interval_default = 180
        elif plugin["frequency"] == "low":
            interval = [3600, 7200, 14400, 21600, 28800, 43200, 86400]
            interval_default = 3600
            retry_interval = [900, 1800, 3600, 7200, 14400]
            retry_interval_default = 1800
        else:
            interval = [
                30, 60, 180, 300, 600, 900, 1800, 3600,
                7200, 14400, 21600, 28800, 43200, 86400]
            interval_default = 60
            retry_interval = [30, 60, 120, 180, 300, 600, 900, 1800, 3600, 7200, 14400]
            retry_interval_default = 60
        v = Validator(post_validation=self._validate_satellites)
        v.set({
            "service_name": {
                "type": str,
                "minlength": 1,
                "maxlength": 100,
                "default": plugin["abstract"]
            },
            "host_alive_check": {
                "type": int,
                "options": [0, 1],
                "default": 0
            },
            "description": {
                "type": str,
                "minlength": 0,
                "maxlength": 100,
                "optional": True,
                "default": "",
            },
            "interval": {
                "type": int,
                "options": interval,
                "default": interval_default,
            },
            "retry_interval": {
                "type": int,
                "options": retry_interval,
                "default": retry_interval_default,
            },
            "notification_interval": {
                "type": int,
                "options": [0, 300, 600, 900, 1800, 3600, 7200, 21600, 43200, 86400],
                "default": 0
            },
            "attempt_max": {
                "type": int,
                "options": list(range(0, 21)),
                "default": 3
            },
            "fd_enabled": {
                "type": int,
                "options": [0,1],
                "default": 1
            },
            "fd_time_range": {
                "type": int,
                "options": [900, 1800, 3600, 7200, 14400],
                "default": 1800
            },
            "fd_flap_count": {
                "type": int,
                "options": list(range(1, 21)),
                "default": 6
            },
        })

        if ignore_missing == True:
            v.set({
                "command_options": {
                    "type": dict,
                    "optional": True,
                },
            })
        else:
            v.set({
                "command_options": {
                    "type": dict,
                    "default": {},
                },
            })

        # 0 = localhost
        # 1 = satellites, bloonix server or localhost
        # 2 = bloonix server or localhost
        # 3 = bloonix server only
        if plugin["network_service"] in [1, 2]:
            remote_check = {
                "type": int,
                "options": [0, 1],
            }
            if "default_location" in plugin and plugin["default_location"] == "remote":
                remote_check["default"] = 1
            else:
                remote_check["default"] = 0
            v.set({"remote_check": remote_check})

            if plugin.get("network_service", 0) == 1:
                v.set({
                    "satellite_type": {
                        "type": int,
                        "options": [0,1],
                        "default": 0,
                        "optional": True,
                    },
                    "satellites": {
                        "type": list,
                        "multioptions": self.schema.satellite.get_ids(),
                        "allow_empty": True,
                        "default": [],
                        "optional": True,
                    }
                })
        elif plugin["network_service"] == 3:
            v.set({
                "remote_check": {
                    "type": int,
                    "options": [1],
                    "default": 1
                }
            })
        else:
            v.set({
                "remote_check": {
                    "type": int,
                    "options": [0],
                    "default": 0
                }
            })

        # table service
        if service_type == "service":
            v.set({
                "enabled": {
                    "type": int,
                    "options": [0, 1],
                    "default": 1
                },
                "acknowledged": {
                    "type": int,
                    "options": [0, 1],
                    "default": 0
                },
                "notification_enabled": {
                    "type": int,
                    "options": [0, 1],
                    "default": 1
                },
                "comment": {
                    "type": str,
                    "minlength": 0,
                    "maxlength": 400,
                    "default": "",
                }
            });

        return v

    def _validate_comman_options_metric_condition(self, value, metrics):
        if (not isinstance(value, list)
                or len(value) != 3
                or value[0] not in metrics
                or value[1] not in ["lt", "le", "gt", "ge", "eq", "ne"]
                or not isinstance(value[2], (str, int, float))):
            return False
        return True

    def _validate_comman_options_string_condition(self, value, strings):
        if (not isinstance(value, list)
                or len(value) != 3
                or value[0] not in strings
                or value[1] not in ["eq", "ne", "re", "rn"]
                or not isinstance(value[2], str)):
            return False
        return True

    def _validate_comman_options_json_condition(self, value):
        if (not isinstance(value, list)
                or len(value) != 3
                or not isinstance(value[0], str)
                or not re.search(r"\w", value[0])
                or value[1] not in ["lt", "le", "gt", "ge", "eq", "ne", "re", "rn"]
                or not isinstance(value[2], str)):
            return False
        return True

    def _validate_command_options(self, res, plugin):
        command_options = res.data["command_options"]
        """
        command_options = {
          "http_status_code": "200",
          "timeout": "15",
          "data_urlencode": [],
          "user_agent": "Bloonix Plugin Check-HTTP",
          "certificate": "21,14",
          "http_header": [],
          "notice": [
            ["time_connect", "gt", 10],
            ["time_connect", "gt", 20]
          ],
          "warning": [],
          "alert": [],
          "critical": [],
          "match_notice": [],
          "match_warning": [],
          "match_alert": [],
          "match_critical": [],
          "workflow": [],   <--- wtrm only
        }
        """
        v = Validator(no_mixin=plugin["options_no_mixin"])
        for param, opts in plugin["options"].items():
            rule = {}
            if plugin["command"] == "check-wtrm" and param == "workflow":
                rule["function"] = _validate_wtrm_workflow
            if opts["datatype"] == "string_condition":
                rule["type"] = list
                rule["multiple"] = True
                rule["function"] = self._validate_comman_options_string_condition
                rule["function_args"] = [list(plugin["strings"])]
            elif opts["datatype"] == "metric_condition":
                rule["type"] = list
                rule["multiple"] = True
                rule["function"] = self._validate_comman_options_metric_condition
                rule["function_args"] = [list(plugin["metrics"])]
            elif opts["datatype"] == "json_condition":
                rule["type"] = list
                rule["multiple"] = True
                rule["function"] = self._validate_comman_options_json_condition
            elif opts["datatype"] == "list" or opts["datatype"] == "wtrm":
                rule["type"] = list
            elif opts["datatype"] == "json-kv":
                rule["type"] = dict
            elif opts["datatype"] == "bool":
                rule["type"] = bool
            elif opts["datatype"] in ["int", "float", "str", "lstr", "json-raw"]:
                rule["type"] = str
            else:
                H.log.error(
                    "unhandled datatype found: %s (%s)"
                    % (opts["datatype"], plugin["command"]))
                H.log.error(plugin, dump=True)
                raise Exception(
                    "unhandled datatype found: %s (%s)"
                    % (opts["datatype"], plugin["command"]))
            for key in ["multiple", "default", "optional"]:
                if key in opts and opts[key] is not None:
                    rule[key] = opts[key]
            v.set({param: rule})
        for key in list(command_options):
            if (command_options[key] is None or command_options[key] is False
                    or (isinstance(command_options[key], str)
                        and command_options[key] == "")):
                del command_options[key]
        r = v.validate(command_options, ignore_missing=False)
        for key in r.failed:
            res.failed["command_options:%s" % key] = r.failed[key]
        if len(res.failed):
            res.has_failed = True

    def _validate_satellites(self, data):
        if "remote_check" in data and data["remote_check"] == 0:
            data["satellite_type"] = 0
            data["satellites"] = []
            return

        if "satellite_type" in data and data["satellite_type"] == 0:
            data["satellites"] = []
            return

        if "satellites" in data:
            if len(data["satellites"]) == 0:
                return {"satellites": "err-716"}
            if len(data["satellites"]) < 3:
                return {"satellites": "err-706"}

    def by_template_id(self, template_id):
        stmt, bind = self.sql.select(
            table=[
                "service_parameter", [
                    "id", "service_name", "command", "remote_check",
                    "interval", "retry_interval", "attempt_max"
                ]
            ],
            condition=["where", ["service_parameter.template_id", template_id]],
            order=["asc", "service_name"]
        )
        return self.dbi.fetchall(stmt, bind)

    def create_service_template(self, data):
        return self.dbi.run_transaction(
            {
                "service_parameter": "write",
                "service": "write",
                "host_template": "write"
            },
            self.__create_service_template,
            data
        )

    def __create_service_template(self, data):
        host_templates = self.schema.host_template.fetchall(
            columns="host_id",
            condition=["where", ["template_id", data["template_id"]]])

        if data["command"] == "check-wtrm":
            wtrm = True
        else:
            wtrm = None
        self.schema.service.check_count_services(plus=len(host_templates), wtrm=wtrm)
        service_parameter = self.create_and_get(data);

        while len(host_templates):
            data = []
            while len(host_templates) and len(data) < 1000:
                row = host_templates.pop()
                data.append({
                    "enabled": 1,
                    "acknowledged": 0,
                    "notification_enabled": 1,
                    "service_parameter_id": service_parameter["id"],
                    "host_id": row["host_id"]
                })
            self.schema.service.create(data)
        return service_parameter

    def create_service(self, data, host_id):
        if data["command"] == "check-wtrm":
            wtrm = True
        else:
            wtrm = None
        self.schema.service.check_count_services(wtrm=wtrm)
        return self.dbi.run_transaction(
            {"service_parameter": "write", "service": "write"},
            self.__create_service,
            data, host_id)

    def __create_service(self, data, host_id):
        result = {}
        service_data = {"host_id": host_id}

        for key in ["enabled", "acknowledged", "notification_enabled", "comment"]:
            if key in data:
                service_data[key] = data[key]
                del data[key]

        ret = self.create_and_get(data)
        result["service_parameter"] = {"data": ret}
        service_data["service_parameter_id"] = ret["id"]
        ret = self.schema.service.create_and_get(service_data)
        result["service"] = {"data": ret}
        return result

    def update_service(self, data, service_parameter_id, service_id):
        return self.dbi.run_transaction(
            {"service_parameter": "write", "service": "write"},
            self.__update_service,
            data, service_parameter_id, service_id)

    def __update_service(self, data, service_parameter_id, service_id):
        service_data = {}

        for key in ["enabled", "acknowledged", "notification_enabled", "comment",]:
            if key in data:
                service_data[key] = data[key]
                del data[key]

        self.update_by_id(service_parameter_id, data)
        if len(service_data):
            self.schema.service.update_by_id(service_id, service_data)

        return True
