from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix import ccodes
from bloonix import rx
from bloonix import utils

class Template(CRUDV):
    def __init__(self):
        self.has_unique_column = {
            "column": "name",
            "err": "err-618",
        }
        self.validator = Validator()
        self.validator.set({
            "name": {
                "type": str,
                "minlength": 1,
                "maxlength": 100,
            },
            "tags": {
                "type": list,
                "maxsize": 20,
                "maxlength": 1000,
                "default": [],
                "regex": r"\w",
                "allow_empty": True
            },
            "description": {
                "type": str,
                "minlength": 0,
                "maxlength": 1000,
                "default": ""
            },
            "variables": {
                "type": dict,
                "function": utils.validate_pv_dict,
                "maxlength": 24000,
                "default": {},
                "allow_empty": True
            }
        })

    def by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=["template", ["id", "name"]],
            join=[
                "inner", [
                    "host_template",
                    "template.id",
                    "host_template.template_id"
                ]
            ],
            condition=[
                "where", ["host_template.host_id", host_id]
            ],
            order=["asc", "template.name"]
        )
        return self.dbi.fetchall(stmt, bind)

    def get_names_by_ids(self, ids):
        templates = {}
        stmt, bind = self.sql.select(
            table=[
                "template", ["id", "name"]
            ],
            condition=[
                "where", ["id", "in", ids]
            ])
        for t in self.dbi.fetchall(stmt, bind):
            templates[t["id"]] = t["name"]
        return templates
