import time
from bloonix.heaven import H

class Service:
    def __init__(self):
        self.dbs = H.model.db.service
        self.dbsp = H.model.db.service_parameter

    def auto(self, opts):
        command = opts.get("command", None)

        # /config/hosts/:id/services/:service_id
        if "service_id" in opts:
            H.cache["service_type"] = "service"
            H.cache["service"] = self.dbs.get(opts["service_id"])
            if not H.cache["service"]:
                return H.plugin.error.set("err-600")
            H.cache["service_parameter"] = self.dbsp.get(
                H.cache["service"]["service_parameter_id"])
            if not H.cache["service_parameter"]:
                return H.plugin.error.set("err-600")
            command = H.cache["service_parameter"]["command"]

        # /config/templates/:id/services/:service_parameter_id
        if "service_parameter_id" in opts:
            H.cache["service_parameter"] = self.dbsp.get(
                opts["service_parameter_id"])
            if not H.cache["service_parameter"]:
                return H.plugin.error.set("err-600")
            command = H.cache["service_parameter"]["command"]

        if command is not None:
            H.cache["plugin"] = H.model.db.plugin.get(command, "command")
            if not H.cache["plugin"]:
                return H.plugin.error.set("err-600")
            # H.cache has either "host" or "template" from auto()
            if "template" in H.cache:
                H.cache["service_type"] = "service_parameter"
            elif "host" in H.cache:
                H.cache["service_type"] = "service"
            else:
                return H.plugin.error.set("err-500")

        return True

    def show(self, opts):
        self.options(opts)
        H.stash.data = H.stash.data["values"]

    def options(self, opts):
        options, defaults = self.dbsp.get_form_values(
            H.cache["service_type"], H.cache["plugin"])
        H.stash.data["options"] = options
        H.stash.data["variables"] = {}
        H.stash.data["plugin"] = H.cache["plugin"]

        if "template" in H.cache:
            H.stash.data["variables"].update(H.cache["template"]["variables"])

        if "host" in H.cache:
            H.stash.data["variables"].update(H.cache["host"]["variables"])

        if "service_id" in opts or "service_parameter_id" in opts:
            H.stash.data["values"] = H.cache["service_parameter"]
            if "service_id" in opts:
                for k in ["enabled", "acknowledged", "notification_enabled", "comment"]:
                    H.stash.data["values"][k] = H.cache["service"][k]
            if "command_options" in H.stash.data["values"]:
                for key, value in H.stash.data["values"]["command_options"].items():
                    H.stash.data["values"]["command_options:%s" % key] = value
                del H.stash.data["values"]["command_options"]
        else:
            H.stash.data["values"] = defaults
            plugin_options = H.stash.data["plugin"]["options"]
            for key in plugin_options:
                if ("default" in plugin_options[key]
                        and plugin_options[key]["default"] is not None):
                    H.stash.data["values"]["command_options:%s" % key] = plugin_options[key]["default"]

    def create(self, opts):
        if not H.plugin.token.check():
            return

        plugin = H.cache["plugin"]
        res = self.dbsp.validate(H.cache["service_type"], H.cache["plugin"])
        res.data["command"] = plugin["command"]

        if "command_options" in res.data:
            for key in list(res.data["command_options"]):
                if res.data["command_options"][key] is None:
                    del res.data["command_options"][key]

        if len(res.failed) > 0:
            return H.plugin.error.set("err-610", failed=res.failed)

        if "host" in H.cache:
            ret = self.dbsp.create_service(res.data, opts["id"])
            if ret is None:
                return H.plugin.error.set("err-604")
            H.stash.data = ret["service_parameter"]["data"]
            H.stash.data["enabled"] = ret["service"]["data"]["enabled"]
            H.plugin.audit.create(
                target="service_parameter",
                new_data=H.stash.data)
        elif "template" in H.cache:
            res.data["template_id"] = opts["id"]
            ret = self.dbsp.create_service_template(res.data)
            if ret is None:
                return H.plugin.error.set("err-604")
            H.stash.data = ret
            H.plugin.audit.create(
                target="service_parameter",
                new_data=res.data)

    def update(self, opts):
        if not H.plugin.token.check():
            return

        res = self.dbsp.validate(
            H.cache["service_type"],
            H.cache["plugin"],
            ignore_missing=True)

        if len(res.failed) > 0:
            return H.plugin.error.set("err-610", failed=res.failed)

        if "command_options" in res.data:
            # enable this maybe with an extra argument?
            #data = H.cache["service_parameter"]["command_options"]
            #data.update(res.data["command_options"])
            data = res.data["command_options"]
            for key in list(data):
                if data[key] is None:
                    del data[key]
            res.data["command_options"] = data

        if ("interval" in res.data
                and res.data["interval"] < H.cache["service_parameter"]["interval"]):
            res.data["interval_changed_at"] = int(time.time())

        if "host" in H.cache:
            ret = self.dbsp.update_service(
                res.data,
                H.cache["service_parameter"]["id"],
                opts["service_id"])
            if ret is None:
                return H.plugin.error.set("err-604")
            H.plugin.audit.create(
                target="service_parameter",
                old_data=H.cache["service_parameter"],
                new_data=ret)
        elif "template" in H.cache:
            self.dbsp.update_by_id(
                opts["service_parameter_id"],
                res.data)
            H.plugin.audit.create(
                target="service_parameter",
                old_data=H.cache["service_parameter"],
                new_data=res.data)

    def delete(self, opts):
        if not H.plugin.token.check():
            return

        if "host" in H.cache:
            if H.cache["service_parameter"]["template_id"] is None:
                self.dbsp.delete_by_id(H.cache["service_parameter"]["id"])
            else:
                self.dbs.delete_by_id(H.cache["service"]["id"])
                H.plugin.audit.delete(
                    target="service_parameter",
                    old_data=H.cache["service_parameter"])
                H.plugin.audit.delete(
                    target="service",
                    old_data=H.cache["service"])
        elif "template" in H.cache:
            self.dbsp.delete_by_id(opts["service_parameter_id"])
            H.plugin.audit.delete(
                target="service_parameter",
                old_data=H.cache["service_parameter"])
